/*
 * tilt.cpp
 *
 *  Created on: Jun 20, 2024
 *      Author: Dario
 */

#include <sta/math/algorithms/attitude/tilt.hpp>
#include <sta/math/utils.hpp>


namespace sta
{
	namespace math
	{
		Quaternion getTiltFromAcceleration(float ax, float ay, float az)
		{
			float norm = std::sqrt(ax*ax + ay*ay + az*az);

			if (norm != 0)
			{
				ax /= norm;
				ay /= norm;
				az /= norm;
			}

			float theta = std::atan2(ay, az);
			float phi 	= std::atan2(-ax, std::sqrt(ay*ay + az*az));

			Quaternion tilt(
				  std::cos(phi / 2.0f) * std::cos(theta / 2.0f),
				  std::sin(phi / 2.0f) * std::cos(theta / 2.0f),
				  std::cos(phi / 2.0f) * std::sin(theta / 2.0f),
				- std::sin(phi / 2.0f) * std::sin(theta / 2.0f)
			);

			return tilt;
		}
	}  // namespace math
}  // namespace sta


